]> Shamusworld >> Repos - rmac/blobdiff - rmac.c
Added in DSP fixups to sect.c, misc. fixes for 6502 assembler.
[rmac] / rmac.c
diff --git a/rmac.c b/rmac.c
index a4b6b42f74e5c9b37c1117dcd2c48d8902e54cd2..ecc3adcdb51294eab38b80859044f2f68c117bed 100644 (file)
--- a/rmac.c
+++ b/rmac.c
@@ -1,7 +1,7 @@
 //
 // RMAC - Reboot's Macro Assembler for all Atari computers
 // RMAC.C - Main Application Code
-// Copyright (C) 199x Landon Dyer, 2011-2017 Reboot and Friends
+// Copyright (C) 199x Landon Dyer, 2011-2019 Reboot and Friends
 // RMAC derived from MADMAC v1.07 Written by Landon Dyer, 1986
 // Source utilised with the kind permission of Landon Dyer
 //
@@ -33,12 +33,14 @@ int glob_flag;                                      // Assume undefined symbols are global
 int lsym_flag;                                 // Include local symbols in object file
 int sbra_flag;                                 // Warn about possible short branches
 int prg_flag;                                  // !=0, produce .PRG executable (2=symbols)
+int prg_extend;                                        // !=0, output extended .PRG symbols
 int legacy_flag;                               // Do stuff like insert code in RISC assembler
 int obj_format;                                        // Object format flag
 int debug;                                             // [1..9] Enable debugging levels
 int err_flag;                                  // '-e' specified
 int err_fd;                                            // File to write error messages to
 int rgpu, rdsp;                                        // Assembling Jaguar GPU or DSP code
+int robjproc;                                  // Assembling Jaguar Object Processor code
 int dsp56001;                                  // Assembling DSP 56001 code
 int list_fd;                                   // File to write listing to
 int regbank;                                   // RISC register bank
@@ -54,6 +56,16 @@ int activecpu = CPU_68000;           // Active 68k CPU (68000 by default)
 int activefpu = FPU_NONE;              // Active FPU (none by default)
 
 
+//
+// Convert a string to uppercase
+//
+void strtoupper(char * s)
+{
+       while (*s)
+               *s++ &= 0xDF;
+}
+
+
 //
 // Manipulate file extension.
 //
@@ -161,11 +173,14 @@ void DisplayHelp(void)
                "                    o5: Absolute long base displacement to word (off)\n"
                "                    o6: Null branches to NOP                    (off)\n"
                "                    o7: clr.l Dx to moveq #0,Dx                 (off)\n"
+               "                    o8: adda.w/l #x,Dy to addq.w/l #x,Dy        (off)\n"
+               "                    o9: adda.w/l #x,Dy to lea x(Dy),Dy          (off)\n"
                "  ~o[value]         Turn a specific optimisation off\n"
                "  +oall             Turn all optimisations on\n"
                "  ~oall             Turn all optimisations off\n"
                "  -p                Create an ST .prg (without symbols)\n"
                "  -ps               Create an ST .prg (with symbols)\n"
+               "  -px               Create an ST .prg (with exsymbols)\n"
                "                    Forces -fa\n"
                "  -r[size]          Pad segments to boundary size specified\n"
                "                    w: word (2 bytes, default alignment)\n"
@@ -194,7 +209,7 @@ void DisplayVersion(void)
                "| |  | | | | | | (_| | (__ \n"
                "|_|  |_| |_| |_|\\__,_|\\___|\n"
                "\nReboot's Macro Assembler\n"
-               "Copyright (C) 199x Landon Dyer, 2011-2017 Reboot\n"
+               "Copyright (C) 199x Landon Dyer, 2011-2018 Reboot\n"
                "V%01i.%01i.%01i %s (%s)\n\n", MAJOR, MINOR, PATCH, __DATE__, PLATFORM);
 }
 
@@ -267,6 +282,7 @@ int Process(int argc, char ** argv)
        err_flag = 0;                                   // Initialise error flag
        rgpu = 0;                                               // Initialise GPU assembly flag
        rdsp = 0;                                               // Initialise DSP assembly flag
+       robjproc = 0;                                   // Initialise OP assembly flag
        lsym_flag = 1;                                  // Include local symbols in object file
        regbank = BANK_N;                               // No RISC register bank specified
        orgactive = 0;                                  // Not in RISC org section
@@ -319,7 +335,7 @@ int Process(int argc, char ** argv)
                                }
 
                                sy->sattr = DEFINED | EQUATED | ABS;
-                               sy->svalue = (*s ? (uint32_t)atoi(s) : 0);
+                               sy->svalue = (*s ? (uint64_t)atoi(s) : 0);
                                break;
                        case 'e':                               // Redirect error message output
                        case 'E':
@@ -428,7 +444,8 @@ int Process(int argc, char ** argv)
                        case 'P':
                                /*
                                 * -p           .PRG generation w/o symbols
* -ps  .PRG generation with symbols
                                 */
                                switch (argv[argno][2])
                                {
@@ -441,6 +458,11 @@ int Process(int argc, char ** argv)
                                                prg_flag = 2;
                                                break;
 
+                                       case 'x':
+                                       case 'X':
+                                               prg_flag = 3;
+                                               break;
+
                                        default:
                                                printf("-p: syntax error\n");
                                                errcnt++;
@@ -428,7 +444,8 @@ int Process(int argc, char ** argv)
                                /*
                                 * -p           .PRG generation w/o symbols
-                                * -ps  .PRG generation with symbols
+                                * -ps          .PRG generation with symbols
+                                * -px          .PRG generation with extended symbols
                                 */
                                switch (argv[argno][2])